package com.miaxis.topic.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.utils.bean.BeanUtils;
import com.miaxis.goods.domain.GoodsInfo;
import com.miaxis.goods.service.IGoodsInfoService;
import com.miaxis.topic.domain.TopicInfo;
import com.miaxis.topic.dto.TopicInfoDto;
import com.miaxis.topic.mapper.TopicInfoMapper;
import com.miaxis.topic.service.ITopicInfoService;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 专题Service业务层处理
 *
 * @author miaxis
 * @date 2021-03-11
 */
@Service
@RequiredArgsConstructor
public class TopicInfoServiceImpl extends ServiceImpl<TopicInfoMapper, TopicInfo> implements ITopicInfoService {

    private final TopicInfoMapper topicInfoMapper;

    private final IGoodsInfoService goodsInfoService;

    /**
     * 查询专题列表
     *
     * @param topicInfo 专题
     * @return 专题
     */
    @Override
    public List<TopicInfo> selectTopicInfoList(TopicInfo topicInfo){
        return topicInfoMapper.selectTopicInfoList(topicInfo);
    }

    /**
     * 新增专题
     * @param topicInfoDto
     * @return
     */
    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public Response<Integer> addTopic(TopicInfoDto topicInfoDto) {

        //保存商品信息
        List<GoodsInfo> goodsInfoList = topicInfoDto.getGoodsInfoList();
        goodsInfoService.saveBatch(goodsInfoList);

        //保存专题信息
        TopicInfo topicInfo = new TopicInfo();
        BeanUtils.copyProperties(topicInfoDto,topicInfo);
        String s1 = goodsInfoList.stream()
                .map(g -> g.getId() + ",")
                .collect(Collectors.toList())
                .toString()
                .replace("[","")
                .replace("]","");
        topicInfo.setGoodsId(s1);
        this.save(topicInfo);

        return Response.success();
    }

    /**
     * 删除专题
     * @param ids
     * @return
     */
    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public Response<Integer> removeTopicByIds(Long[] ids) {

        for (Long id : ids) {
            //首先删除专题对应商品
            String goodsIds = this.getById(id).getGoodsId();
            if (goodsIds.indexOf(",") != -1){
                for (String goodsId : Arrays.asList(goodsIds.split(","))) {
                    goodsInfoService.update(new UpdateWrapper<GoodsInfo>().set("status",1).eq("id",goodsId));
                }
            }else {
                goodsInfoService.update(new UpdateWrapper<GoodsInfo>().set("status",1).eq("id",goodsIds));
            }
            //删除专题
            this.update(new UpdateWrapper<TopicInfo>().set("status",1).eq("id",id));
        }
        return Response.success();
    }

}
